
function demo_BEZIER_definedomain_02(domain_type)

%--------------------------------------------------------------------------
% OBJECT:
%--------------------------------------------------------------------------
% Demo illustrating:
% 1. how to define a BEZIER composite boundary, using arcs of disks,
%    ellipses, segments, polygons and "free BEZIER". The routines work on
%    piecewise BEZIER of different order.
%    To this purpose see the MATLAB function "define_domain" at the
%    bottom of this demo.
% 2. how to call the indomain routine "cubRS".
%
% This demo is more complicated than
%
% 1. demo_BEZIER_definedomain_00
% 2. demo_BEZIER_definedomain_01
%
% but better illustrates the performance of the codes, on many subdomains
% (see attached file "define_domain" at about line 87).
%--------------------------------------------------------------------------
% Dates
%--------------------------------------------------------------------------
% First version: November 13, 2021;
% Checked: November 14, 2021.
%--------------------------------------------------------------------------

% ------------------------------ Settings ---------------------------------

% The variable "domain_type" set the domain taken into account. The
% invented names are just given to remember something of the region.
%
%       0. Disk
%       1. Lune like
%       2. Tau-like-symbol
%       3. Quadrangloid
%       4. Curved trapezoid
%       5. Weird eta
%       6. Weird eta 2
%       7. Weird eta 3
%       8. Weird eta skinny
%       9. Rough ball
%      10. L-shaped
%      11. M-shaped domain
%      12. defined by a disk, an ellipse and a segment
%      13. defined by a disk, an ellipse, a segment and a free BEZIER
%          (variable order)
%      otherwise: defined by a disk, an ellipse, a segment and a free BEZIER
%          (variable order)

if nargin < 1, domain_type=13; end






% ----------------------------- Main code ---------------------------------

% 1. Make BEZIER structure (see function "define_domain" defined below).
[geometry_BEZIER,domain_str]=define_domain(domain_type);

% 2. Plot figure.
h=figure(1);
f1=ishandle(h)&&strcmp(get(h,'type'),'figure');
if f1,clf(1);end
figure(1)
hold on;
axis equal;
plotNURBSPL(geometry_BEZIER);
hold off;










%--------------------------------------------------------------------------
% ATTACHED ROUTINES
%--------------------------------------------------------------------------

function [geometry_BEZIER ,domain_str]=define_domain(example)

switch example
    
    case 0 % circle
        domain_str='circle';
        P=[1 0; 1 1; 0 1; -1 1; -1 0; -1 -1; 0 -1; 1 -1;  1 0];
        knots=[0 0 0 .25 .25 .5 .5 .75 .75 1 1 1];
        c=1/sqrt(2); w=[1 c 1 c 1 c 1 c 1];
        order=3;
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
        
    case 1 % lune-like
        domain_str='lune-like';
        P=[1 0; 0.45 0.37; 1 1; 0 1; -0.8 0.9; -1 0; 0.3 0.5; 0 -1; ...
            1 -1;  1 0];
        knots=[0 0 0 .15 .25 .25 .5 .5 .75 .75 1 1 1];
        c=1/sqrt(2); w=[1 c c 1 c 1 c 1 c 1];
        order=3;
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
        
    case 2 % tau-like-symbol
        domain_str='tau-like-symbol';
        P=[1 0; 0.45 0.37; 1 1; 0 1; -0.8 0.9; -1 0; 0.3 0.5;  ...
            0.2 -0.45; -0.4 -0.4; 0 -1; 1 -1;  1 0];
        knots=[0 0 0 .15 .25 .25 .5 .5 .6 .7 .75 .75 1 1 1];
        c=1/sqrt(2); w=[1 c c 1 c 1 c 1 c 1 c 1];
        order=3;
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
        
    case 3 % cubic-domain
        domain_str='cubic-domain: leaf';
        P=[1 0; 1 1; 0 1; -1 1; -1 0; -1 -1; 0 -1; 1 -1;  1 0];
        knots=[0 0 0 0 .25 .25 .25 .5 .75 1 1 1 1];
        c=1/sqrt(2); w=[1 c 1 c 1 c 1 c 1];
        order=4;
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
        
        
    case 4 % cubic-domain: nut
        domain_str='cubic-domain: slanted nut';
        P=[1 0; 1 1; 0 1; -1 0.6; -1 0; -1 -1; 0 -1; 1 -1;  1 0];
        knots=[0 0 0 0 .25 .25 .25 .25 .5 1 1 1 1];
        c=1/sqrt(2); w=[1 c 1 c 1 c 1 c 1];
        order=4;
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
        
    case 5 % cubic-domain: slanted boomerang
        domain_str='cubic-domain: golf club';
        P=[1 0; 1 1; 0 1;  -1 0;  -1 -1; -0.4 0.6; 0 0.5; 1 -1;  1 0];
        knots=[0 0 0 0 .25 .25 .25 .35 .65 1 1 1 1];
        c=1/sqrt(2); w=[1 c/2 1 c/4 1 c/4 1 c 1];
        order=4;
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
    case 6 % cubic-domain: nut
        domain_str='cubic-domain: dino head';
        P=[1 0; 1 1; 0 1; -0.4 0.6; -1 0; -1 -1; 0 -1; 0 0.5; 1 -1;  1 0];
        knots=[0 0 0 0 .25 .25 .25 .35 .65 0.75 1 1 1 1];
        c=1/sqrt(2); w=[1 c/2 1 c/4 1 c/4 1 1 c 1];
        order=4;
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
        
    case 7 % cubic-domain: slanted boomerang
        domain_str='cubic-domain: tadpole';
        P=[1 0; 1 1; 0 1;  -1 0;  -1 -1; -0.4 0.6; 0 -0.5; 1 -1;  1 0];
        knots=[0 0 0 0 .25 .3 .35 .45 .65 1 1 1 1];
        c=1/sqrt(2); w=[1 1 1 1 1 1 1 1 1];
        order=4;
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
    case 8 % cubic-domain: nut
        domain_str='cubic-domain: nose';
        P=[1 0; 1 1; 0 1; -0.4 0.6; -1 0; -1 -1; 0 0.5; 1 -1;  1 0];
        knots=[0 0 0 0 .25 .25 .25 .35 .65 1 1 1 1];
        c=1/sqrt(2); w=[1 5 5 5 5 5 5 5 1];
        order=4;
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
    case 9 % rough-ball
        domain_str='rough ball';
        order=3;
        sidesL=20;
        t=linspace(0,2*pi,sidesL); t=t';
        P=[cos(t) sin(t)]; P=[P(1:end-1,:); P(1,:)];
        knots_mid=linspace(0.1,0.9,sidesL-order);
        knots=[zeros(1,order) knots_mid ones(1,order)];
        M=size(P,1);
        % w=rand(1,M);
        w=[ 8.258169774895474e-01
            5.383424352600571e-01
            9.961347166268855e-01
            7.817552875318368e-02
            4.426782697754463e-01
            1.066527701805844e-01
            9.618980808550537e-01
            4.634224134067444e-03
            7.749104647115024e-01
            8.173032206534330e-01
            8.686947053635097e-01
            8.443584551091032e-02
            3.997826490988965e-01
            2.598704028506542e-01
            8.000684802243075e-01
            4.314138274635446e-01
            9.106475944295229e-01
            1.818470283028525e-01
            2.638029165219901e-01
            1.455389803847170e-01]';
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
    case 10 % L-shaped domain
        
        domain_str='L shaped domain';
        P=[-1 1; -1 -1; 1 -1; 1 -0.6; -0.6 -0.6; -0.6 1; -1 1]; % 7 points
        knots=[0 0 0 .15 .25 .35 .65 1 1 1];
        w=[0.2 5 10 10 100 5 0.2];
        order=3;
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
    case 11 % M-shaped domain
        
        domain_str='M shaped domain';
        order=3;
        P=[-1 1; -1 0; -1 -1; -0.6 -1; -0.6 0; -0.6 0.6; 0 0; 0.6 0.6; ...
            0.6 0; 0.6 -1; 1 -1; 1 0; 1 1; 0.6 1; 0 0.4; -0.6 1; -1 1];
        kk=size(P,1)-order;
        knots_mid=linspace(0.1,0.9,kk);
        knots=[0 0 0 knots_mid 1 1 1]; % 12-3
        w=1000*ones(1,size(P,1));
        
        geometry_BEZIER =makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);
        
    case 12 % variable order
        domain_str='domain 12, defined by a disk, an ellipse and a segment';
        
        
        % add arc of a disk
        geometry_BEZIER(1)=makeBEZIERarc('disk_arc',...
            'center',[0 0],'angles',[0 pi/2],'radius',1);
        
        % compute first point of the piecewise BEZIER domain
        Pinit=firstpointNURBSPL(geometry_BEZIER(1));
        
        % compute last point of the so made BEZIER
        Pend=lastpointNURBSPL(geometry_BEZIER(1));
        
        % add arc of an ellipse
        geometry_BEZIER(2)=makeBEZIERarc('elliptical_arc',...
            'center',Pend-[1 0],'angles',[0 pi+pi/4],...
            'ell_axis',[1 2],'tilt_angle',0);
        
        % compute last point of the so made BEZIER
        Pend=lastpointNURBSPL(geometry_BEZIER(2));
        
        % "close" the boundary with a segment
        geometry_BEZIER(3)=makeBEZIERarc('segment','extrema',...
            [Pend; Pinit]);
        
        % join piecewise BEZIER
        geometry_BEZIER=joinNURBSPLarcs(geometry_BEZIER);
        
        
    case 13 % variable order
        
        domain_str1='domain 13, defined by a disk, an ellipse, a segment';
        domain_str2=' and a free NURBS (variable order)';
        domain_str=strcat(domain_str1,domain_str2);
        
        % add arc of a disk
        geometry_BEZIER (1)=makeBEZIERarc('disk_arc',...
            'center',[0 0],'angles',[0 pi/2],'radius',1);
        
        % compute first point of the piecewise NURBS domain
        Pinit=firstpointNURBSPL(geometry_BEZIER (1));
        
        % compute last point of the so made NURBS
        Pend=lastpointNURBSPL(geometry_BEZIER (end));
        
        % add arc of an ellipse
        geometry_BEZIER (2)=makeBEZIERarc('elliptical_arc',...
            'center',Pend-[1 0],'angles',[0 pi+pi/4],'ell_axis',[1 2],...
            'tilt_angle',0);
        
        % compute last point of the so made NURBS
        Pend=lastpointNURBSPL(geometry_BEZIER (end));
        
        % add segment
        geometry_BEZIER (3)=makeBEZIERarc('segment','extrema',[Pend; 0 Pend(2)]);
        
        % compute first point of the piecewise NURBS domain
        Pinit=firstpointNURBSPL(geometry_BEZIER );
        
        % "close" the boundary with a "free" NURBS.
        geometry_BEZIER (4)=makeBEZIERarc('free',...
            'P',[0 Pend(2); -1.9 0.3; -1.8 0.5; Pinit],...
            'knots',[0 0 0 0 1 1 1 1],'weights',[1 1 2 1],'order',4);
        
        % join piecewise NURBS
        geometry_BEZIER =joinNURBSPLarcs(geometry_BEZIER );
        
        
        
    otherwise % variable order
        domain_str1='domain defined by a disk, an ellipse, a segment';
        domain_str2=' and a free NURBS (variable order)';
        domain_str=strcat(domain_str1,domain_str2);
        
        % add arc of a disk
        geometry_BEZIER (1)=makeNURBSarc('disk_arc',...
            'center',[0 0],'angles',[0 pi/2],'radius',1);
        
        % compute first point of the piecewise NURBS domain
        Pinit=firstpointNURBSPL(geometry_BEZIER (1));
        
        % compute last point of the so made NURBS
        Pend=lastpointNURBSPL(geometry_BEZIER (end));
        
        % add arc of an ellipse
        geometry_BEZIER (2)=makeNURBSarc('elliptical_arc',...
            'center',Pend-[1 0],'angles',[0 pi+pi/4],'ell_axis',[1 2],...
            'tilt_angle',0);
        
        % compute last point of the so made NURBS
        Pend=lastpointNURBSPL(geometry_BEZIER (end));
        
        % add segment
        geometry_BEZIER (3)=makeNURBSarc('segment','extrema',[Pend; 0 Pend(2)]);
        
        % compute first point of the piecewise NURBS domain
        Pinit=firstpointNURBSPL(geometry_BEZIER );
        
        % "close" the boundary with a "free" NURBS.
        geometry_BEZIER (4)=makeNURBSarc('free',...
            'P',[0 Pend(2); -1.9 0.3; -1.8 0.5; Pinit],...
            'knots',[0 0 0 0.5 1 1 1],'weights',[1 1 2 1],'order',3);
        
        % join piecewise NURBS
        geometry_BEZIER =joinNURBSPLarcs(geometry_BEZIER );
end




