
function demo_nurbs_test_04(domain_type)

%--------------------------------------------------------------------------
% OBJECT:
%--------------------------------------------------------------------------
% Demo illustrating CUBATURE:
% 1. how to define a NURBS on a composite boundary, using arcs of disks,
%    ellipses, segments, polygons and "free NURBS". The routines work on
%    piecewise NURBS of different order.
%    To this purpose see the MATLAB function "define_domain" at the
%    bottom of this demo.
% 2. how to get an algebraic cubature rule on the desired domain, via
%    "cubRS" routine.
% 3. testing the routines for specific integrands:
%        f1=@(x,y) exp(-(x.^2+y.^2));
%        f2=@(x,y) (x.^2+(y-0.4).^2).^(11/2);
%        f3=@(x,y) (x.^2+(y-0.4).^2).^(1/2);
%
% Note: In the numerical experiments of the accompanying paper, we have
%       used
%       1. S(3): M domain, choose domain_type=1,
%       2. S(4): joining arcs of a disk, ellipse, segment, domain_type=2,
%       3. S(5): concave domain, domain_type=3.
%--------------------------------------------------------------------------
% INPUT:
%--------------------------------------------------------------------------
% domain_type: scalar that refers to a domain of integration (see routine
%              "define_domain" attached below);
%--------------------------------------------------------------------------
% Reference paper:
%--------------------------------------------------------------------------
% A. Sommariva and M. Vianello,
% ﻿Low cardinality Positive Interior quadrature on NURBS-shaped domains.
%--------------------------------------------------------------------------
% Dates
%--------------------------------------------------------------------------
% First version: November 13, 2021;
% Checked: December 8, 2021.
%--------------------------------------------------------------------------

warning off;

% ------------------------------ Settings ---------------------------------

% Algebraic degree of precision of the cubature rule. Suggested values are
% mild, e.g. not larger than "ade=15". Typical "ade" in the applications
% are between "3" and "5".
adeV=2:2:10;

% Cubature extraction method: the variable "extraction_type" sets how
% Tchakaloff rule is extracted.
%      1: lsqnonneg, 2: lawsonhanson, 3: LHDM,  4: QR with pivoting.
% If not declared or equal to "[]", the default is LHDM, see
% "dCATCH: a numerical package for d-variate near G-optimal Tchakaloff
% regression via fast NNLS"
% M. Dessole, F. Marcuzzi and M. Vianello
% MDPI-Mathematics 8 (2020) - Special Issue "Numerical Methods"
extraction_type=3;

% Nbox is a technical parameter for indomain routine; in doubt set 100.
Nbox=100;

% In indomain routine one must be certain that points are inside the
% domain. In our cubature needs we just wants some points in the domain.
% Consequently, if we are not fully sure that a point is in the domain, due
% to geometrical issues, we do not take it into account.
safe_mode=0;

% The variable "domain_type" set the domain taken into account. 
%       1. S(3): M domain, choose domain_type=11,
%       2. S(4): joining arcs of a disk, ellipse, segment, domain_type=14,
%       3. S(5): concave domain, domain_type=1.

if nargin < 1, domain_type=1; end






% ----------------------------- Main code ---------------------------------

%--------------------------------------------------------------------------
% 1. Make NURBS structure
%--------------------------------------------------------------------------
[geometry_NURBS,domain_str]=define_domain(domain_type);

REV=[]; logerrV=[];

% Define functions

f1=@(x,y) exp(-(x.^2+y.^2));
f2=@(x,y) (x.^2+(y-0.4).^2).^(11/2);
f3=@(x,y) (x.^2+(y-0.4).^2).^(1/2);

% Define high order rule

[inside,in_doubts,sp_xnV,sp_xdV,sp_ynV,sp_ydV,boxVx,...
    singular_points_X,singular_points_Y]=inRS([],geometry_NURBS,...
    Nbox,safe_mode);
xywL = cubRS(30,geometry_NURBS,0,Nbox,safe_mode);
xxL=xywL(:,1); yyL=xywL(:,2); wwL=xywL(:,3);
I(1)=wwL'*feval(f1,xxL,yyL);
I(2)=wwL'*feval(f2,xxL,yyL);
I(3)=wwL'*feval(f3,xxL,yyL);

fprintf('\n \t ------------------------------------------------------- '); 
fprintf('\n \t            TESTING CUBATURE RELATIVE ERRORS                '); 
fprintf('\n \t ------------------------------------------------------- '); 
fprintf('\n \t * '); disp(domain_str);
fprintf('\n \t \t -----------------------------');
fprintf('\n \t \t ADE    f1      f2      f3'); 
fprintf('\n \t \t -----------------------------');

for jj=1:length(adeV)

    ade=adeV(jj);

    %----------------------------------------------------------------------
    % 2. Compute algebraic cubature rule of the domain (several tests!).
    %----------------------------------------------------------------------
    xyw = cubRS(ade,geometry_NURBS,extraction_type,Nbox,safe_mode);
    xx=xyw(:,1); yy=xyw(:,2); ww=xyw(:,3);

    %----------------------------------------------------------------------
    % 3. Making tests.
    %----------------------------------------------------------------------
    I0(1)=ww'*feval(f1,xx,yy);
    I0(2)=ww'*feval(f2,xx,yy);
    I0(3)=ww'*feval(f3,xx,yy);
    RE=abs((I-I0)./I);

    %----------------------------------------------------------------------
    % 4. Statistics.
    %----------------------------------------------------------------------
    fprintf('\n \t \t %2.0f | %1.0e | %1.0e | %1.0e |',ade,RE(1),...
        RE(2),RE(3));

end

fprintf('\n \t \t ----------------------------------- \n');

fprintf('\n \t ------------------------------------------------------- '); 
fprintf('\n \t                    DESCRIPTION ');
fprintf('\n \t ------------------------------------------------------- '); 
fprintf('\n \t Rule relative error for the following functions: \n');
fprintf('\n \t f1=@(x,y) exp(-(x.^2+y.^2))');
fprintf('\n \t f2=@(x,y) (x.^2+(y-0.4).^2).^(11/2)');
fprintf('\n \t f3=@(x,y) (x.^2+(y-0.4).^2).^(1/2)');
fprintf('\n \t ------------------------------------------------------- '); 
fprintf('\n \n')









%--------------------------------------------------------------------------
% ATTACHED ROUTINES
%--------------------------------------------------------------------------

function [geometry_NURBS,domain_str]=define_domain(example)

switch example

    case 1 % M-shaped domain

        domain_str='S(3): M shaped domain';
        order=3;
        P=[-1 1; -1 0; -1 -1; -0.6 -1; -0.6 0; -0.6 0.6; 0 0; 0.6 0.6; ...
            0.6 0; 0.6 -1; 1 -1; 1 0; 1 1; 0.6 1; 0 0.4; -0.6 1; -1 1];
        kk=size(P,1)-order;
        knots_mid=linspace(0.1,0.9,kk);
        knots=[0 0 0 knots_mid 1 1 1]; % 12-3
        w=1000*ones(1,size(P,1));

        geometry_NURBS=makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);

    case 2

        domain_str='S(4): domain defined by a disk, an ellipse, a segment';
        geometry_NURBS(1)=makeNURBSarc('disk_arc',...
            'center',[0 0],'angles',[0 pi/2],'radius',1);

        % add arc of an ellipse
        geometry_NURBS(2)=makeNURBSarc('elliptical_arc',...
            'center',[0 0],'angles',[pi/2 3*pi/2],'ell_axis',[0.5 1],...
            'tilt_angle',0);

        % add a final segment
        Pend=lastpointNURBSPL(geometry_NURBS(2));
        Pinit=firstpointNURBSPL(geometry_NURBS(1));
        geometry_NURBS(3)=makeNURBSarc('segment','vertices',[Pend; Pinit]);

        % join pieces
        geometry_NURBS=joinNURBSPLarcs(geometry_NURBS);

    case 3 % lune-like
        domain_str='S(5): lune-like';
        P=[1 0; 0.45 0.37; 1 1; 0 1; -0.8 0.9; -1 0; 0.3 0.5; 0 -1; ...
            1 -1;  1 0];
        knots=[0 0 0 .15 .25 .25 .5 .5 .75 .75 1 1 1];
        c=1/sqrt(2); w=[1 c c 1 c 1 c 1 c 1];
        order=3;

        geometry_NURBS=makeNURBSarc('free','P',...
            P,'knots',knots,'weights',w,'order',order);

end




