
function demo_SPLINE_cubature_01

%--------------------------------------------------------------------------
% OBJECT:
%--------------------------------------------------------------------------
% Demo illustrating the CUBATURE procedure:
% 1. how to define a SPLINE on a composite boundary, using defining 
%    directly the spline structure by the function "makeSPLINEarc" if 
%
%    a. control points,
%    b. knots 
%    c. order
%
%    are known.
%    The result is saved in the variable "geometry_SPLINE" of just one
%    component that take into account control points, knots, weights and
%    orders.
% 2. the application of the "joinNURBSPLarcs" is important in case the 
%    length of the vector "geometry_SPLINE" is larger than one, in order to
%    make a "consistent" piecewise SPLINE in which knots are well-ordered; 
%    since in this case there are more arcs, this routine is called.
%--------------------------------------------------------------------------
% Dates
%--------------------------------------------------------------------------
% First version: November 13, 2021;
% Checked: November 16, 2021.
%--------------------------------------------------------------------------

clear all; clf;

% ------------------------------ Settings ---------------------------------

% Algebraic degree of precision of the cubature rule. Suggested values are
% mild, e.g. not larger than "ade=15". Typical "ade" in the applications
% are between "3" and "5".
ade=5;


% ......................... Design SPLINE pieces  .........................

% add arc of a disk
fprintf('\n \t * Defining arc of a disk');
geometry_SPLINE(1)=makeSPLINEarc('disk_arc','center',[0 0],...
    'angles',[0 pi/2],'radius',1);

% compute last point of the so made SPLINE
fprintf('\n \t * Computing last point of a SPLINE');
Pend=lastpointNURBSPL(geometry_SPLINE);

% add arc of an ellipse
fprintf('\n \t * Defining arc of an ellipse');
geometry_SPLINE(2)=makeSPLINEarc('elliptical_arc',... 
    'center',Pend-[1 0],'angles',[0 pi+pi/4],'ell_axis',[1 2],...
    'tilt_angle',0);

% compute last point of the so made SPLINE
fprintf('\n \t * Computing last point of a SPLINE');
Pend=lastpointNURBSPL(geometry_SPLINE);

% add segment
fprintf('\n \t * Defining a polygonal_arc');
geometry_SPLINE(3)=makeSPLINEarc('polygonal_arc',...
    'extrema',[Pend; 0 Pend(2)]);

% compute first point of the piecewise SPLINE domain
fprintf('\n \t * Computing first point of a SPLINE');
Pinit=firstpointNURBSPL(geometry_SPLINE);

P=[0 Pend(2); -1.5 0.3; -1.3 0.5; Pinit];
knots=linspace(0,1,size(P,1));
geometry_SPLINE(4)=makeSPLINEarc('free','P',P,'order',3);

% Join SPLINE pieces make unique SPLINE structure (very important step!)
fprintf('\n \t * Joining piecewise SPLINE into one well def. structure');
structure_SPLINE=joinNURBSPLarcs(geometry_SPLINE);

%--------------------------------------------------------------------------
% 1. Cubature.
%--------------------------------------------------------------------------
xyw = cubRS(ade,structure_SPLINE);

%--------------------------------------------------------------------------
% 2. Plot domain and control points polygon.
%--------------------------------------------------------------------------

h=figure(1);
f1=ishandle(h)&&strcmp(get(h,'type'),'figure');
if f1,clf(1);end
figure(1)
hold on;
axis equal;

% A. plot SPLINE curve (black)
plotNURBSPL(structure_SPLINE);

% B. plot nodes.
plot(xyw(:,1),xyw(:,2),'mo','MarkerEdgeColor','k',...
    'MarkerFaceColor','m','MarkerSize',10);

hold off;

fprintf('\n \t ------------------------------------------- \n \t');
fprintf('\n \t            SPLINE CUBATURE TESTS \n \t');
fprintf('\n \t ------------------------------------------- \n \t');
fprintf('\n');
fprintf('\n \t ALGEBRAIC DEGREE OF PRECISION: %3.0f',ade);
w=xyw(:,3); condcub=sum(abs(w))/sum(w);
fprintf('\n \t CUBATURE CONDITIONING        : %1.5e',condcub);
NN=length(find(xyw(:,3) == 0));
fprintf('\n \t NEGATIVE WEIGHTS             : %6.0f',NN);
fprintf('\n \t ------------------------------------------- \n \t');
fprintf('\n \t             PLOT DESCRIPTION \n \t');
fprintf('\n \t ------------------------------------------- \n \t');
fprintf('\n \t * Magenta dots: quadrature nodes.   \n \t');
fprintf('\n \t ------------------------------------------- \n');



