
function demo_RQ_00

%--------------------------------------------------------------------------
% OBJECT:
%--------------------------------------------------------------------------
% The purpose of this demo is to compare the results of several quadrature 
% routines for integrating rational functions "f=p/q" in [-1,1], being "p"
% "q" polynomials.
%
% We suppose that 
% 1. no zero of "q" belongs to "[-1,1]",
% 2. the zeros of "q"  may be complex and with multiplicity higher than 1.
%--------------------------------------------------------------------------

% ............................ settings ...................................
deg=30;
x0=1.00001;
mult0=10;
f=@(x) (3+x).^deg./(x-x0).^mult0;

% define symbolic integrand
syms t; syms g;
g=(3+t)^deg/(t-x0)^mult0;

% ............................ main code ..................................

% .......... CHEBFUN .......... 
tic;
F=chebfun(f,'splitting','on');
S(1)=sum(F);
cpus(1)=toc;


% .......... RFEJER .......... 
tic;
denmat=[x0 mult0];
sgl=encodeRFejer(deg,denmat);
[xRF,ErrRF,RelRF,yRF]=rfejer(sgl);
x=(xRF(1,:))'; w=(xRF(2,:))';
fx=feval(f,x);
S(2)=w'*fx;
cpus(2)=toc;

% .......... GAUSS ..........
tic;
XW=legendre_rules(20000); x=XW(:,1); w=XW(:,2);
fx=feval(f,x);
S(3)=w'*fx;
cpus(3)=toc;

%  .......... SYMS ..........

Ssymb=vpa(int(g,-1,1));



% ....................... results and statistics ..........................
fprintf('\n \t ....................... Settings ........................');
fprintf('\n \t Numerator Degree: %3.0f',deg);
fprintf('\n \t Denominator Root: %1.15e',x0);
fprintf('\n \t Denominator Mult: %3.0f',mult0);

fprintf('\n \t ....................... Results ........................');
fprintf('\n \t CHEBFUN: %1.15e',S(1));
fprintf('\n \t RFEJER : %1.15e',S(2));
fprintf('\n \t GAULEG : %1.15e',S(3));
fprintf('\n \t SYMB.  : %1.15e',Ssymb);

fprintf('\n \t ....................... Errors .........................');
fprintf('\n \t CHEBFUN vs RFEJER AE: %1.15e',abs(S(1)-S(2)));
if abs(S(2)) > 0, 
  fprintf('\n \t CHEBFUN vs RFEJER RE: %1.15e',abs(S(1)-S(2))/abs(S(2)));  
end


fprintf('\n \t GAUSS vs RFEJER AE : %1.1e',abs(S(3)-S(2)));
if abs(S(2)) > 0, 
  fprintf('\n \t GAUSS vs RFEJER RE : %1.1e',abs(S(3)-S(2))/abs(S(2)));  
end



fprintf('\n \t GAUSS vs CHEBFUN AE : %1.1e',abs(S(3)-S(1)));
if abs(S(2)) > 0, 
  fprintf('\n \t GAUSS vs CHEBFUN RE : %1.1e',abs(S(3)-S(1))/abs(S(1)));  
end

fprintf('\n');

fprintf('\n \t CHEBFUN vs SYMB AE: %1.15e',abs(S(1)-Ssymb));
if abs(Ssymb) > 0, 
  fprintf(2,'\n \t CHEBFUN vs SYMB RE: %1.1e',abs(S(1)-Ssymb)/abs(Ssymb));  
end

fprintf('\n \t RFEJER vs SYMB AE: %1.15e',abs(S(2)-Ssymb));
if abs(Ssymb) > 0, 
  fprintf(2,'\n \t RFEJER vs SYMB RE: %1.1e',abs(S(2)-Ssymb)/abs(Ssymb));  
end

fprintf('\n \t GAUSS vs SYMB AE : %1.1e',abs(S(3)-S(2)));
if abs(S(2)) > 0, 
  fprintf(2,'\n \t GAUSS vs SYMB RE : %1.1e',abs(S(3)-S(2))/abs(S(2)));  
end

fprintf('\n \t ....................... Cputime ........................');
fprintf('\n \t CHEBFUN: %1.15e',cpus(1));
fprintf('\n \t RFEJER : %1.15e',cpus(2));
fprintf('\n \t GAUSS  : %1.15e',cpus(3));
fprintf('\n \n');


%  	 ....................... Settings ........................
%  	 Numerator Degree:  10
%  	 Denominator Root: 1.000100000000000e+00
%  	 Denominator Mult:  10
%  	 ....................... Results ........................
%  	 CHEBFUN: 1.164988879442321e+41
%  	 RFEJER : 1.165048036717373e+41
%  	 GAULEG : 1.165048036727232e+41
%  	 SYMB.  : 1.165048036725802e+41
%  	 ....................... Errors .........................
%  	 CHEBFUN vs RFEJER AE: 5.915727505190171e+36
%  	 CHEBFUN vs RFEJER RE: 5.077668318173611e-05
%  	 GAUSS vs RFEJER AE : 9.9e+29
%  	 GAUSS vs RFEJER RE : 8.5e-12
%  	 GAUSS vs CHEBFUN AE : 5.9e+36
%  	 GAUSS vs CHEBFUN RE : 5.1e-05
%  	 CHEBFUN vs SYMB AE: 5.915728348115859e+36
%  	 CHEBFUN vs SYMB RE: 5.1e-05
%  	 RFEJER vs SYMB AE: 8.429256885222337e+29
%  	 RFEJER vs SYMB RE: 7.2e-12
%  	 GAUSS vs SYMB AE : 9.9e+29
%  	 GAUSS vs SYMB RE : 8.5e-12
%  	 ....................... Cputime ........................
%  	 CHEBFUN: 3.734308300000000e-02
%  	 RFEJER : 4.232000000000000e-03
%  	 GAUSS  : 1.471583000000000e-03