
function demo_RQ_02(example)

%--------------------------------------------------------------------------
% OBJECT:
%--------------------------------------------------------------------------
% The purpose of this demo is to compare the results of several quadrature
% routines for integrating rational functions "f=p/q" in [-1,1], being "p"
% "q" polynomials.
%
% We suppose that
% 1. no zero of "q" belongs to "[-1,1]",
% 2. the zeros of "q"  may be complex and with multiplicity higher than 1.
%
% In this demo, we have may have:
%      real multiple roots,
%      multiple conjugate complex zeros.
%--------------------------------------------------------------------------

% ............................ settings ...................................

if nargin < 1, example=1; end
switch example
    case 1
        % RFEJER mildly failing, relative error: 10^(-10)
        deg=0;
        x0=1.1+i*10^(-4); x1=conj(x0); mult0=2; mult1=mult0;
        f=@(t) (1+t).^deg./((t-x0).^mult0 .*(t-x1).^mult1);
        % define symbolic integrand
        syms t; syms g;
        g=(1+t)^deg/((t-x0)^mult0*(t-x1)^mult1);
    case 2
        % RFEJER, INTEGRAL ok, Chebfun, Gauss Legendre (10000 pts) fail.
        deg=0;
        x0=i*10^(-4); x1=conj(x0); mult0=6; mult1=mult0;
        f=@(t) (1+t).^deg./((t-x0).^mult0 .*(t-x1).^mult1);
        % define symbolic integrand
        syms t; syms g;
        g=(1+t)^deg/((t-x0)^mult0*(t-x1)^mult1);
    case 3
        % Everything fails (even INTEGRAL). Odd: is it SYMBOLIC ERROR?
        deg=0;
        x0=1.1+i*10^(-4); x1=conj(x0); mult0=8; mult1=mult0;
        f=@(t) (1+t).^deg./((t-x0).^mult0 .*(t-x1).^mult1);
        % define symbolic integrand
        syms t; syms g;
        g=(1+t)^deg/((t-x0)^mult0*(t-x1)^mult1);
    case 4
        % RFEJER mild fail. CHEBFUN, GAUSS fail. INTEGRAL ok.
        deg=6;
        x0=i*10^(-4); x1=conj(x0); mult0=4; mult1=mult0;
        f=@(t) (1+t).^deg./((t-x0).^mult0 .*(t-x1).^mult1);
        % define symbolic integrand
        syms t; syms g;
        g=(1+t)^deg/((t-x0)^mult0*(t-x1)^mult1);

    case 5
        % RFEJER mild fail. CHEBFUN, GAUSS fail. INTEGRAL ok.
        deg=6;
        x0=i*10^(-4); x1=conj(x0); mult0=4; mult1=mult0;
        f=@(t) (t).^deg./((t-x0).^mult0 .*(t-x1).^mult1);
        % define symbolic integrand
        syms t; syms g;
        g=(t)^deg/((t-x0)^mult0*(t-x1)^mult1);

  case 6 
      % CHEBFUN, RFEJER mild fail, GAUSS, INTEGRAL 10^(-13). INTEGRAL
      % warning.
        deg=30;
        x0=1.00001; x1=-1.01; mult0=10; mult1=20;
        f=@(x) (3+x).^deg./((x-x0).^mult0 .* (x-x1).^mult1);
        syms t; syms g; g=(3+t)^deg/((t-x0)^mult0 .* (t-x1)^mult1);
    case 7
        deg=12;
        x0=1.00001; x1=+1.01; mult0=10; mult1=20;
        f=@(x) (3+x).^deg./((x-x0).^mult0 .* (x-x1).^mult1);
        syms t; syms g; g=(3+t)^deg/((t-x0)^mult0 .* (t-x1)^mult1);

end




% ............................ main code ..................................

% .......... CHEBFUN ..........
tic;
F=chebfun(f,'splitting','on');
S(1)=sum(F);
cpus(1)=toc;


% .......... RFEJER ..........
tic;
denmat=[x0 mult0; x1 mult1];
sgl=encodeRFejer(deg,denmat);
[xRF,ErrRF,RelRF,yRF]=rfejer(sgl);
x=(xRF(1,:))'; w=(xRF(2,:))';
fx=feval(f,x);
S(2)=w'*fx;
cpus(2)=toc;

% .......... GAUSS ..........
tic;
XW=legendre_rules(10000); x=XW(:,1); w=XW(:,2);
% [x,w]=legpts(10000); w=w';
fx=feval(f,x);
S(3)=w'*fx;
cpus(3)=toc;

% .......... QUADL ..........
tic;
% XW=legendre_rules(10000); x=XW(:,1); w=XW(:,2);
S(4) = integral(f,-1,1,'AbsTol',1e-14,'RelTol',1e-14);
cpus(4)=toc;

%  .......... SYMS ..........

Ssymb=vpa(int(g,-1,1));



% ....................... results and statistics ..........................
fprintf('\n \t ....................... Settings ........................');
fprintf('\n \t Numerator Degree: %3.0f',deg);
fprintf('\n \t Denominator real(Root 1): %1.15e',real(x0));
fprintf('\n \t Denominator imag(Root 1): %1.15e',imag(x0));
fprintf('\n \t Denominator Mult 1: %3.0f',mult0);
fprintf('\n \t Denominator real(Root 2): %1.15e',real(x1));
fprintf('\n \t Denominator imag(Root 2): %1.15e',imag(x1));
fprintf('\n \t Denominator Mult 2: %3.0f',mult1);

fprintf('\n \t ....................... Results ........................');
fprintf('\n \t CHEBFUN  : %1.15e',S(1));
fprintf('\n \t RFEJER   : %1.15e',S(2));
fprintf('\n \t GAULEG   : %1.15e',S(3));
fprintf('\n \t INTEGRAL : %1.15e',S(4));
fprintf('\n \t SYMB.    : %1.15e',Ssymb);

fprintf('\n \t ....................... Errors .........................');
fprintf('\n \t CHEBFUN vs RFEJER AE: %1.15e',abs(S(1)-S(2)));
if abs(S(2)) > 0,
    fprintf('\n \t CHEBFUN vs RFEJER RE: %1.15e',abs(S(1)-S(2))/abs(S(2)));
end


fprintf('\n \t GAUSS vs RFEJER AE : %1.1e',abs(S(3)-S(2)));
if abs(S(2)) > 0,
    fprintf('\n \t GAUSS vs RFEJER RE : %1.1e',abs(S(3)-S(2))/abs(S(2)));
end



fprintf('\n \t GAUSS vs CHEBFUN AE : %1.1e',abs(S(3)-S(1)));
if abs(S(2)) > 0,
    fprintf('\n \t GAUSS vs CHEBFUN RE : %1.1e',abs(S(3)-S(1))/abs(S(1)));
end

fprintf('\n');

fprintf('\n \t CHEBFUN vs SYMB AE: %1.1e',abs(S(1)-Ssymb));
if abs(Ssymb) > 0,
    fprintf(2,'\n \t CHEBFUN vs SYMB RE: %1.1e',abs(S(1)-Ssymb)/abs(Ssymb));
end

fprintf('\n \t RFEJER vs SYMB AE: %1.1e',abs(S(2)-Ssymb));
if abs(Ssymb) > 0,
    fprintf(2,'\n \t RFEJER vs SYMB RE: %1.1e',abs(S(2)-Ssymb)/abs(Ssymb));
end

fprintf('\n \t GAUSS vs SYMB AE : %1.1e',abs(S(3)-Ssymb));
if abs(Ssymb) > 0,
    fprintf(2,'\n \t GAUSS vs SYMB RE : %1.1e',abs(S(3)-Ssymb)/abs(Ssymb));
end

fprintf('\n \t INTEG vs SYMB AE : %1.1e',abs(S(4)-Ssymb));
if abs(Ssymb) > 0,
    fprintf(2,'\n \t INTEG vs SYMB RE : %1.1e',abs(S(4)-Ssymb)/abs(Ssymb));
end

fprintf('\n \t ....................... Cputime ........................');
fprintf('\n \t CHEBFUN: %1.15e',cpus(1));
fprintf('\n \t RFEJER : %1.15e',cpus(2));
fprintf('\n \t GAUSS  : %1.15e',cpus(3));
fprintf('\n \t INTEG  : %1.15e',cpus(4));
fprintf('\n \n');




% EXAMPLE 1:
%  	 ....................... Settings ........................
%  	 Numerator Degree:   0
%  	 Denominator real(Root 1): 1.100000000000000e+00
%  	 Denominator imag(Root 1): 1.000000000000000e-04
%  	 Denominator Mult 1:   2
%  	 Denominator real(Root 2): 1.100000000000000e+00
%  	 Denominator imag(Root 2): -1.000000000000000e-04
%  	 Denominator Mult 2:   2
%  	 ....................... Results ........................
%  	 CHEBFUN  : 3.332969401005868e+02
%  	 RFEJER   : 3.332969398141590e+02
%  	 GAULEG   : 3.332969401005865e+02
%  	 INTEGRAL : 3.332969401005868e+02
%  	 SYMB.    : 3.332969401005877e+02
%  	 ....................... Errors .........................
%  	 CHEBFUN vs RFEJER AE: 2.864278485503746e-07
%  	 CHEBFUN vs RFEJER RE: 8.593773729518255e-10
%  	 GAUSS vs RFEJER AE : 2.9e-07
%  	 GAUSS vs RFEJER RE : 8.6e-10
%  	 GAUSS vs CHEBFUN AE : 2.8e-13
%  	 GAUSS vs CHEBFUN RE : 8.5e-16
% 
%  	 CHEBFUN vs SYMB AE: 8.8e-13
%  	 CHEBFUN vs SYMB RE: 2.6e-15
%  	 RFEJER vs SYMB AE: 2.9e-07
%  	 RFEJER vs SYMB RE: 8.6e-10
%  	 GAUSS vs SYMB AE : 1.2e-12
%  	 GAUSS vs SYMB RE : 3.5e-15
%  	 INTEG vs SYMB AE : 8.8e-13
%  	 INTEG vs SYMB RE : 2.6e-15
%  	 ....................... Cputime ........................
%  	 CHEBFUN: 1.526058300000000e-02
%  	 RFEJER : 1.329762500000000e-02
%  	 GAUSS  : 4.589144580000000e-01
%  	 INTEG  : 2.888042000000000e-03

% EXAMPLE 2:
%  	 ....................... Settings ........................
%  	 Numerator Degree:   0
%  	 Denominator real(Root 1): 0.000000000000000e+00
%  	 Denominator imag(Root 1): 1.000000000000000e-04
%  	 Denominator Mult 1:   6
%  	 Denominator real(Root 2): 0.000000000000000e+00
%  	 Denominator imag(Root 2): -1.000000000000000e-04
%  	 Denominator Mult 2:   6
%  	 ....................... Results ........................
%  	 CHEBFUN  : 3.666987931168178e+43
%  	 RFEJER   : 7.731263170956529e+43
%  	 GAULEG   : 3.616795358137884e+41
%  	 INTEGRAL : 7.731263170943632e+43
%  	 SYMB.    : 7.731263170943632e+43
%  	 ....................... Errors .........................
%  	 CHEBFUN vs RFEJER AE: 4.064275239788351e+43
%  	 CHEBFUN vs RFEJER RE: 5.256935574326738e-01
%  	 GAUSS vs RFEJER AE : 7.7e+43
%  	 GAUSS vs RFEJER RE : 1.0e+00
%  	 GAUSS vs CHEBFUN AE : 3.6e+43
%  	 GAUSS vs CHEBFUN RE : 9.9e-01
% 
%  	 CHEBFUN vs SYMB AE: 4.1e+43
%  	 CHEBFUN vs SYMB RE: 5.3e-01
%  	 RFEJER vs SYMB AE: 1.3e+32
%  	 RFEJER vs SYMB RE: 1.7e-12
%  	 GAUSS vs SYMB AE : 7.7e+43
%  	 GAUSS vs SYMB RE : 1.0e+00
%  	 INTEG vs SYMB AE : 3.0e+27
%  	 INTEG vs SYMB RE : 3.8e-17
%  	 ....................... Cputime ........................
%  	 CHEBFUN: 3.325650000000000e-02
%  	 RFEJER : 4.978583000000000e-03
%  	 GAUSS  : 1.018333000000000e-03
%  	 INTEG  : 3.380375000000000e-03

% EXAMPLE 3:
%  	 ....................... Settings ........................
%  	 Numerator Degree:   0
%  	 Denominator real(Root 1): 1.100000000000000e+00
%  	 Denominator imag(Root 1): 1.000000000000000e-04
%  	 Denominator Mult 1:   8
%  	 Denominator real(Root 2): 1.100000000000000e+00
%  	 Denominator imag(Root 2): -1.000000000000000e-04
%  	 Denominator Mult 2:   8
%  	 ....................... Results ........................
%  	 CHEBFUN  : 6.666619608032518e+13
%  	 RFEJER   : 2.401668447093403e+36
%  	 GAULEG   : 6.666619608032522e+13
%  	 INTEGRAL : 6.666619608032523e+13
%  	 SYMB.    : 4.431542033332568e+19
%  	 ....................... Errors .........................
%  	 CHEBFUN vs RFEJER AE: 2.401668447093403e+36
%  	 CHEBFUN vs RFEJER RE: 1.000000000000000e+00
%  	 GAUSS vs RFEJER AE : 2.4e+36
%  	 GAUSS vs RFEJER RE : 1.0e+00
%  	 GAUSS vs CHEBFUN AE : 3.9e-02
%  	 GAUSS vs CHEBFUN RE : 5.9e-16
% 
%  	 CHEBFUN vs SYMB AE: 4.4e+19
%  	 CHEBFUN vs SYMB RE: 1.0e+00
%  	 RFEJER vs SYMB AE: 2.4e+36
%  	 RFEJER vs SYMB RE: 5.4e+16
%  	 GAUSS vs SYMB AE : 4.4e+19
%  	 GAUSS vs SYMB RE : 1.0e+00
%  	 INTEG vs SYMB AE : 4.4e+19
%  	 INTEG vs SYMB RE : 1.0e+00
%  	 ....................... Cputime ........................
%  	 CHEBFUN: 2.311216600000000e-02
%  	 RFEJER : 4.328083000000000e-03
%  	 GAUSS  : 1.115834000000000e-03
%  	 INTEG  : 3.222791000000000e-03

% EXAMPLE 4:
%  	 ....................... Settings ........................
%  	 Numerator Degree:   6
%  	 Denominator real(Root 1): 0.000000000000000e+00
%  	 Denominator imag(Root 1): 1.000000000000000e-04
%  	 Denominator Mult 1:   4
%  	 Denominator real(Root 2): 0.000000000000000e+00
%  	 Denominator imag(Root 2): -1.000000000000000e-04
%  	 Denominator Mult 2:   4
%  	 ....................... Results ........................
%  	 CHEBFUN  : 9.205148586114377e+27
%  	 RFEJER   : 9.817477337003022e+27
%  	 GAULEG   : 4.349971566959111e+26
%  	 INTEGRAL : 9.817477336992415e+27
%  	 SYMB.    : 9.817477336992418e+27
%  	 ....................... Errors .........................
%  	 CHEBFUN vs RFEJER AE: 6.123287508886452e+26
%  	 CHEBFUN vs RFEJER RE: 6.237129252957059e-02
%  	 GAUSS vs RFEJER AE : 9.4e+27
%  	 GAUSS vs RFEJER RE : 9.6e-01
%  	 GAUSS vs CHEBFUN AE : 8.8e+27
%  	 GAUSS vs CHEBFUN RE : 9.5e-01
% 
%  	 CHEBFUN vs SYMB AE: 6.1e+26
%  	 CHEBFUN vs SYMB RE: 6.2e-02
%  	 RFEJER vs SYMB AE: 1.1e+16
%  	 RFEJER vs SYMB RE: 1.1e-12
%  	 GAUSS vs SYMB AE : 9.4e+27
%  	 GAUSS vs SYMB RE : 9.6e-01
%  	 INTEG vs SYMB AE : 3.3e+12
%  	 INTEG vs SYMB RE : 3.3e-16
%  	 ....................... Cputime ........................
%  	 CHEBFUN: 5.693612500000000e-02
%  	 RFEJER : 4.594459000000000e-03
%  	 GAUSS  : 1.427791000000000e-03
%  	 INTEG  : 3.811083000000000e-03

% EXAMPLE 6:
%  	 ....................... Settings ........................
%  	 Numerator Degree:  30
%  	 Denominator real(Root 1): 1.000010000000000e+00
%  	 Denominator imag(Root 1): 0.000000000000000e+00
%  	 Denominator Mult 1:  10
%  	 Denominator real(Root 2): -1.010000000000000e+00
%  	 Denominator imag(Root 2): 0.000000000000000e+00
%  	 Denominator Mult 2:  20
%  	 ....................... Results ........................
%  	 CHEBFUN  : 1.105700260595035e+56
%  	 RFEJER   : 1.105700260567977e+56
%  	 GAULEG   : 1.105700261059535e+56
%  	 INTEGRAL : 1.105700261059712e+56
%  	 SYMB.    : 1.105700261059834e+56
%  	 ....................... Errors .........................
%  	 CHEBFUN vs RFEJER AE: 2.705729379111956e+45
%  	 CHEBFUN vs RFEJER RE: 2.447073113397002e-11
%  	 GAUSS vs RFEJER AE : 4.9e+46
%  	 GAUSS vs RFEJER RE : 4.4e-10
%  	 GAUSS vs CHEBFUN AE : 4.6e+46
%  	 GAUSS vs CHEBFUN RE : 4.2e-10
% 
%  	 CHEBFUN vs SYMB AE: 4.6e+46
%  	 CHEBFUN vs SYMB RE: 4.2e-10
%  	 RFEJER vs SYMB AE: 4.9e+46
%  	 RFEJER vs SYMB RE: 4.4e-10
%  	 GAUSS vs SYMB AE : 3.0e+43
%  	 GAUSS vs SYMB RE : 2.7e-13
%  	 INTEG vs SYMB AE : 1.2e+43
%  	 INTEG vs SYMB RE : 1.1e-13
%  	 ....................... Cputime ........................
%  	 CHEBFUN: 2.493844590000000e-01
%  	 RFEJER : 6.138262500000000e-02
%  	 GAUSS  : 1.991666000000000e-03
%  	 INTEG  : 5.641366700000000e-02

% EXAMPLE 7:
% warning: Minimum step size reached near x = 1. There may be a singularity, or the tolerances may be
% too tight for this problem. 
% > In integralCalc/checkSpacing (line 467)
% In integralCalc/iterateScalarValued (line 319)
% In integralCalc/vadapt (line 132)
% In integralCalc (line 75)
% In integral (line 87)
% In demo_RQ_02 (line 114) 
% 
%  	 ....................... Settings ........................
%  	 Numerator Degree:  12
%  	 Denominator real(Root 1): 1.000010000000000e+00
%  	 Denominator imag(Root 1): 0.000000000000000e+00
%  	 Denominator Mult 1:  10
%  	 Denominator real(Root 2): 1.010000000000000e+00
%  	 Denominator imag(Root 2): 0.000000000000000e+00
%  	 Denominator Mult 2:  20
%  	 ....................... Results ........................
%  	 CHEBFUN  : 1.859481751841885e+91
%  	 RFEJER   : 1.859481750568059e+91
%  	 GAULEG   : 1.859481752464926e+91
%  	 INTEGRAL : 1.859481752465240e+91
%  	 SYMB.    : 1.859481752465476e+91
%  	 ....................... Errors .........................
%  	 CHEBFUN vs RFEJER AE: 1.273825517047206e+82
%  	 CHEBFUN vs RFEJER RE: 6.850433012628710e-10
%  	 GAUSS vs RFEJER AE : 1.9e+82
%  	 GAUSS vs RFEJER RE : 1.0e-09
%  	 GAUSS vs CHEBFUN AE : 6.2e+81
%  	 GAUSS vs CHEBFUN RE : 3.4e-10
% 
%  	 CHEBFUN vs SYMB AE: 6.2e+81
%  	 CHEBFUN vs SYMB RE: 3.4e-10
%  	 RFEJER vs SYMB AE: 1.9e+82
%  	 RFEJER vs SYMB RE: 1.0e-09
%  	 GAUSS vs SYMB AE : 5.5e+78
%  	 GAUSS vs SYMB RE : 3.0e-13
%  	 INTEG vs SYMB AE : 2.4e+78
%  	 INTEG vs SYMB RE : 1.3e-13
%  	 ....................... Cputime ........................
%  	 CHEBFUN: 6.469920799999999e-02
%  	 RFEJER : 4.085808300000000e-02
%  	 GAUSS  : 6.713250000000000e-03
%  	 INTEG  : 2.636691600000000e-02